<?php

declare(strict_types=1);

namespace Drupal\notifier_email_channel;

use Drupal\Core\Config\BootstrapConfigStorageFactory;
use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Reference;
use Symfony\Component\Mailer\Messenger\MessageHandler;

/**
 * Notifier email channel compiler pass.
 */
final class NotifierEmailChannelCompilerPass implements CompilerPassInterface {

  public function process(ContainerBuilder $container): void {
    $found = FALSE;
    foreach ($container->getDefinitions() as $definition) {
      if ($definition->getClass() === MessageHandler::class) {
        $found = TRUE;
      }
    }

    if ($found === FALSE) {
      // Remove messenger when no message handler exists. This is currently the
      // non-Lite Symfony Mailer.
      // @todo remove after https://www.drupal.org/project/symfony_mailer/issues/3394123
      $container->getDefinition('notifier.channel.email')
        ->setArgument('$bus', NULL);
    }

    // If @symfony_mailer_lite.transports exists than wire it in. Non-lite
    // already has it in v2@alpha in mailer_transport.services.yml.
    if ($container->hasDefinition('symfony_mailer_lite.transports')) {
      $container->getDefinition('notifier.channel.email')
        ->setArgument('$transport', new Reference('symfony_mailer_lite.transports'));
    }

    $container->getDefinition('notifier.channel.email')
      ->setArgument('$from', $this->getSettings()['mail']);
  }

  /**
   * Get active configuration.
   *
   * @return array{'mail': string}
   *   The active configuration.
   */
  private function getSettings(): array {
    /** @var array{'mail': string} $settings */
    $settings = BootstrapConfigStorageFactory::get()->read('system.site');
    return ($settings !== FALSE ? $settings : []) + [
      'mail' => '',
    ];
  }

}
