<?php

declare(strict_types=1);

namespace Drupal\symfony_mailer_lite_state_transport\Transport;

use Drupal\Core\State\StateInterface;
use Symfony\Component\Mailer\Exception\UnsupportedSchemeException;
use Symfony\Component\Mailer\SentMessage;
use Symfony\Component\Mailer\Transport\AbstractTransportFactory;
use Symfony\Component\Mailer\Transport\Dsn;
use Symfony\Component\Mailer\Transport\TransportInterface;

final class StateTransportFactory extends AbstractTransportFactory {

  public const STATE_NAME_BUFFER = 'symfony_mailer_lite_state_transport_mail_buffer';

  final public function __construct(
    private StateInterface $state,
  ) {
    parent::__construct();
  }

  public function create(Dsn $dsn): TransportInterface {
    if ('drupal-state' === $dsn->getScheme()) {
      return new StateTransport($this);
    }

    throw new UnsupportedSchemeException($dsn, 'null', $this->getSupportedSchemes());
  }

  /**
   * @phpstan-return list<non-empty-string>
   */
  protected function getSupportedSchemes(): array {
    return ['drupal-state'];
  }

  public function append(SentMessage $message): void {
    /** @var \Symfony\Component\Mailer\SentMessage[] $buffer */
    $buffer = $this->state->get(static::STATE_NAME_BUFFER, []);
    $buffer[] = $message;
    $this->state->set(static::STATE_NAME_BUFFER, $buffer);
  }

  /**
   * Get the email buffer.
   *
   * @param bool $clear
   *   Whether to clear buffer after getting.
   *
   * @return \Symfony\Component\Mailer\SentMessage[]
   *   The messages in the buffer.
   */
  public function getBuffer(bool $clear = FALSE): array {
    /** @var \Symfony\Component\Mailer\SentMessage[] $buffer */
    $buffer = $this->state->get(static::STATE_NAME_BUFFER, []);
    if ($clear === TRUE) {
      $this->state->delete(static::STATE_NAME_BUFFER);
    }
    return $buffer;
  }

}
