<?php

declare(strict_types=1);

namespace Drupal\Tests\notifier_email_channel\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\symfony_mailer_lite\Entity\Transport;
use Drupal\symfony_mailer_lite_state_transport\Transport\StateTransportFactory;
use Symfony\Component\Notifier\Notification\Notification;
use Symfony\Component\Notifier\NotifierInterface;
use Symfony\Component\Notifier\Recipient\EmailRecipientInterface;

final class NotifierEmailChannelTest extends KernelTestBase {

  private const FROM = 'from-test@example.com';

  protected static $modules = [
    'notifier',
    'notifier_email_channel',
    'symfony_mailer_lite',
    'system',
    'symfony_mailer_lite_state_transport',
  ];

  protected function setUp(): void {
    parent::setUp();
    $this->installConfig(['system']);
    $this->config('system.site')
      ->set('mail', static::FROM)
      ->save();

    $id = 'test_transport';
    Transport::create([
      'id' => $id,
      'plugin' => 'dsn',
      'configuration' => [
        'dsn' => 'drupal-state://default',
      ],
    ])->save();

    \Drupal::configFactory()
      ->getEditable('symfony_mailer_lite.settings')
      ->set('default_transport', $id)
      ->save();

    $this->container->get('kernel')->rebuildContainer();
  }

  public function testEmailChannel(): void {
    $recipient = new class implements EmailRecipientInterface {

      public function getEmail(): string {
        return 'notifier-email-channel@example.com';
      }

    };

    $subject = 'Email subject!';
    $contents = 'Email contents!';
    $notification = (new Notification())
      ->subject($subject)
      ->content($contents);

    static::notifier()->send(
      notification: $notification,
      recipients: $recipient,
    );

    $buffer = static::stateTransportFactory()->getBuffer();
    static::assertCount(1, $buffer);

    $message = $buffer[0]->getMessage()->toString();
    static::assertStringContainsString('To: notifier-email-channel@example.com', $message);
    static::assertStringContainsString(\sprintf('From: %s', static::FROM), $message);
    // Body starts after blank line after headers.
    static::assertStringContainsString(\sprintf("\n%s", $contents), $message);
    static::assertStringContainsString($subject, $message);
  }

  private static function notifier(): NotifierInterface {
    /** @var \Symfony\Component\Notifier\NotifierInterface */
    return \Drupal::service(NotifierInterface::class);
  }

  private static function stateTransportFactory(): StateTransportFactory {
    /** @var \Drupal\symfony_mailer_lite_state_transport\Transport\StateTransportFactory */
    return \Drupal::service(StateTransportFactory::class);
  }

}
